# Generated by Django 5.0.8 on 2025-08-27 07:33

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ixp_tracker', '0012_statspercountry_routed_asn_count_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='ixp',
            name='manrs_participant',
            field=models.BooleanField(default=False),
        ),
    ]
