# Generated by Django 5.2.5 on 2025-08-29 14:22

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ixp_tracker', '0016_statspercountry_routed_asns_ixp_member_customers_rate_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='ixp',
            name='org_id',
            field=models.IntegerField(null=True),
        ),
    ]
