# Generated by Django 5.2.5 on 2025-08-27 12:25

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ixp_tracker', '0014_statsperixp_members_joined_last_12_months_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='statsperixp',
            name='monthly_members_change',
            field=models.IntegerField(default=0),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='statsperixp',
            name='monthly_members_change_percent',
            field=models.FloatField(default=0),
            preserve_default=False,
        ),
    ]
